import React, { useState } from 'react'
import GodsVisualization from './components/visualizations/GodsVisualization'
import ArtifactsVisualization from './components/visualizations/ArtifactsVisualization'
import SatelliteEndUsersVisualization from './components/visualizations/SatelliteEndUsersVisualization'
import SatelliteLaunchesVisualization from './components/visualizations/SatelliteLaunchesVisualization'
import AsteroidProfitVisualization from './components/visualizations/AsteroidProfitVisualization'
import MarsMissionVisualization from './components/visualizations/MarsMissionVisualization'
import BlackHolesVisualization from './components/visualizations/BlackHolesVisualization'
import SolarSystemVisualization from './components/visualizations/SolarSystemVisualization'
import NasaPrioritiesVisualization from './components/visualizations/NasaPrioritiesVisualization'
import AmericanInterestVisualization from './components/visualizations/AmericanInterestVisualization'
import SpaceAgenciesVisualization from './components/visualizations/SpaceAgenciesVisualization'
import LaunchesFailuresVisualization from './components/visualizations/LaunchesFailuresVisualization'

const sections = [
    {
        name: '',
        story: 'A Tale of Space and Humanity',
        visualization: () => { return <></> }
    },
    {
        name: 'The Gods of Space',
        story: 'Since the beginning of remembered time, space has always been an object of human fascination and beliefs...',
        visualization: GodsVisualization
    },
    {
        name: 'Historical Artifacts of Astronomy',
        story: 'But it has not only marked our beliefs, but also our technology and architecture for millenia...',
        visualization: ArtifactsVisualization
    },
    {
        name: 'The Solar System',
        story: 'And after slowly discovering the vast presences surrounding us...',
        visualization: SolarSystemVisualization
    },
    {
        name: 'Black Holes',
        story: 'And observing endless mysteries in even vaster spaces...',
        visualization: BlackHolesVisualization
    },
    {
        name: 'Launch Successes and Failures',
        story: 'We eventually began to step into this darkness. Although it didn\'t always succeed.',
        visualization: LaunchesFailuresVisualization
    },
    {
        name: 'Space Agencies',
        story: 'But with growing success and rapidly developing technologies increased the human desire to be a part of this step...',
        visualization: SpaceAgenciesVisualization
    },
    {
        name: 'Satellite Launches',
        story: 'And after discovering the potential, we began populating our orbit with tools.',
        visualization: SatelliteLaunchesVisualization
    },
    {
        name: 'Satellite End Users',
        story: 'Today, the technologies and artifacts we have placed in the surrounding space are a normality and neccessity to our lives.',
        visualization: SatelliteEndUsersVisualization
    },
    {
        name: "People's Interest in Space",
        story: 'Which does not mean that we have stopped dreaming of going ourselves.',
        visualization: AmericanInterestVisualization
    },
    {
        name: "People's Opinion on Nasa Priorities",
        story: 'Although we may not always agree on the next important step.',
        visualization: NasaPrioritiesVisualization
    },
    {
        name: 'Mars Missions',
        story: 'But the journeys do not end here. The future lies in space...',
        visualization: MarsMissionVisualization
    },
    {
        name: 'Asteroid Profit Margins',
        story: 'Even if our gaze has moved from faith to profit.',
        visualization: AsteroidProfitVisualization
    },
    {
        name: '',
        story: 'The End (of this demo, not of humanity or space)',
        visualization: () => { return <a href="/data/m8_dv_ass3_report.pdf" download>Download Report/Sources</a> }
    }
]

export default function Section() {
    const [section, setSection] = useState(0)

    const incrementSection = () => {
        if (section === sections.length - 1)
            return
        setSection(section + 1)
    }

    const decrementSection = () => {
        if (section === 0)
            return
        setSection(section - 1)
    }

    return (
        <>
            <div className='visualization-header'>
                <div>
                    <br />
                    {!(section > 0) ? null :
                        <span
                            className='section-nav'
                            onClick={decrementSection}
                        >
                            Prev
                        </span>
                    }
                    {!(section > 0 && section < sections.length - 1) ? null :
                        <>&nbsp;|&nbsp;</>
                    }
                    {!(section < sections.length - 1) ? null :
                        <span
                            className='section-nav'
                            onClick={incrementSection}
                        >
                            Next
                        </span>
                    }
                </div>
                <h3><i>{sections[section].story}</i></h3>
                <div>{sections[section].name}</div>
            </div>
            {React.createElement(sections[section].visualization, {})}
        </>
    )
}